#include <iostream>
#include <iomanip>
#include <string>
#include "FontRenderer.hpp"
#include "BitStream.hpp"

using namespace std;

void printBuffer(std::ostream& str, const std::vector<uint8_t>& buf)
{
  str << hex << setfill('0');
  for(size_t i = 0; i < buf.size(); ++i)
  {
    cout << "0x" << setw(2) << +buf[i] << ", ";
    if(i % 16 == 15)
      cout << "\n    ";
  }
}

int main(int argc, char** argv)
{
  FontRenderer fr;
  fr.LoadFont(argv[1], stoi(argv[2]));

  // Print font stats
  FontMetrics m = fr.GetMetrics();
  cout << "// ### Font metrics ###" << endl;
  cout << "// Ascender:  " << m.ascender << endl;
  cout << "// Descender: " << m.descender << endl;
  cout << "// Width:     " << m.width << endl << endl;

  // Print types
  cout << "typedef struct" << endl;
  cout << "{" << endl;
  cout << "  uint16_t bufferOffset;" << endl;
  cout << "  uint8_t width;" << endl;
  cout << "  uint8_t height;" << endl;
  cout << "  int8_t x_pos;" << endl;
  cout << "  uint8_t y_pos;" << endl;
  cout << "  uint8_t advance;" << endl;
  cout << "} Glyph;" << endl;
  cout << endl;
  cout << "typedef struct" << endl;
  cout << "{" << endl;
  cout << "  uint8_t maxHeight;" << endl;
  cout << "  Glyph glyphs[0x60];" << endl;
  cout << "  uint8_t buffer[];" << endl;
  cout << "} Font;" << endl;
  cout << endl;

  // Export font
  int height = m.ascender + m.descender;
  std::vector<uint8_t> glyphBuffer;
  cout << "const __flash Font arial8 = \n{" << endl;
  cout << "  " << height << ",\n  {" << endl;
  for(int i = 0x20; i < 0x80; ++i)
  {
    Glyph g = fr.RenderGlyph(i);
    cout << "    {" << glyphBuffer.size() << ", " << g.width << ", " << g.height << ", " << g.xPos << ", " << g.yPos << ", " << g.advance << "}," << endl;
    glyphBuffer.insert(glyphBuffer.end(), g.buffer.begin(), g.buffer.end());
  }
  cout << "  },\n  {\n    ";
  printBuffer(cout, glyphBuffer);
  cout << "\n  }\n};" << endl;
}